@@ -6,7 +6,7 @@
 /* SYNOPSIS
 /*	\fBpostdrop\fR [\fB-rv\fR] [\fB-c \fIconfig_dir\fR]
 /* DESCRIPTION
-/*	The \fBpostdrop\fR command creates a file in the \fBmaildrop\fR
+/*	The \fBpostdrop\fR(1) command creates a file in the \fBmaildrop\fR
 /*	directory and copies its standard input to the file.
 /*
 /*	Options:
@@ -52,7 +52,7 @@
 /*	The following \fBmain.cf\fR parameters are especially relevant to
 /*	this program.
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /* .IP "\fBalternate_config_directories (empty)\fR"
 /*	A list of non-default Postfix configuration directories that may
 /*	be specified with "-c config_directory" on the command line, or
@@ -72,7 +72,12 @@
 /*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
 /* .IP "\fBtrigger_timeout (10s)\fR"
 /*	The time limit for sending a trigger to a Postfix daemon (for
-/*	example, the pickup(8) or qmgr(8) daemon).
+/*	example, the \fBpickup\fR(8) or \fBqmgr\fR(8) daemon).
+/* .PP
+/*	Available in Postfix version 2.2 and later:
+/* .IP "\fBauthorized_submit_users (static:anyone)\fR"
+/*	List of users who are authorized to submit mail with the \fBsendmail\fR(1)
+/*	command (and with the privileged \fBpostdrop\fR(1) helper command).
 /* FILES
 /*	/var/spool/postfix/maildrop, maildrop queue
 /* SEE ALSO
@@ -127,6 +132,7 @@
 #include <cleanup_user.h>
 #include <record.h>
 #include <rec_type.h>
+#include <user_acl.h>
 
 /* Application-specific. */
 
@@ -143,43 +149,61 @@
   * and so on.
   */
 
+ /*
+  * Local mail submission access list.
+  */
+char   *var_submit_acl;
+
+static CONFIG_STR_TABLE str_table[] = {
+    VAR_SUBMIT_ACL, DEF_SUBMIT_ACL, &var_submit_acl, 0, 0,
+    0,
+};
+
  /*
   * Queue file name. Global, so that the cleanup routine can find it when
   * called by the run-time error handler.
   */
 static char *postdrop_path;
 
-/* postdrop_cleanup - callback for the runtime error handler */
-
-static void postdrop_cleanup(void)
-{
-
-    /*
-     * This is the fatal error handler. Don't try to do anything fancy.
-     */
-    if (postdrop_path) {
-	(void) remove(postdrop_path);
-	postdrop_path = 0;
-    }
-}
-
 /* postdrop_sig - catch signal and clean up */
 
 static void postdrop_sig(int sig)
 {
 
     /*
-     * Assume atomic signal() updates, even when emulated with sigaction().
+     * This is the fatal error handler. Don't try to do anything fancy.
+     * 
+     * msg_vstream does not allocate memory, but msg_syslog may indirectly in
+     * syslog(), so it should not be called from a user-triggered signal
+     * handler.
+     * 
+     * Assume atomic signal() updates, even when emulated with sigaction(). We
+     * use the in-kernel SIGINT handler address as an atomic variable to
+     * prevent nested postdrop_sig() calls. For this reason, main() must
+     * configure postdrop_sig() as SIGINT handler before other signal
+     * handlers are allowed to invoke postdrop_sig().
      */
     if (signal(SIGINT, SIG_IGN) != SIG_IGN) {
 	(void) signal(SIGQUIT, SIG_IGN);
 	(void) signal(SIGTERM, SIG_IGN);
 	(void) signal(SIGHUP, SIG_IGN);
-	postdrop_cleanup();
-	_exit(sig);
+	if (postdrop_path) {
+	    (void) remove(postdrop_path);
+	    postdrop_path = 0;
+	}
+	/* Future proofing. If you need exit() here then you broke Postfix. */
+	if (sig)
+	    _exit(sig);
     }
 }
 
+/* postdrop_cleanup - callback for the runtime error handler */
+
+static void postdrop_cleanup(void)
+{
+    postdrop_sig(0);
+}
+
 /* main - the main program */
 
 int     main(int argc, char **argv)
@@ -200,6 +224,7 @@ int     main(int argc, char **argv)
     const char *error_text;
     char   *attr_name;
     char   *attr_value;
+    const char *errstr;
     char   *junk;
 
     /*
@@ -257,6 +282,15 @@ int     main(int argc, char **argv)
      * perform some sanity checks on the input.
      */
     mail_conf_read();
+    get_mail_conf_str_table(str_table);
+
+    /*
+     * Mail submission access control. Should this be in the user-land gate,
+     * or in the daemon process?
+     */
+    if ((errstr = check_user_acl_byuid(var_submit_acl, uid)) != 0)
+	msg_fatal("User %s(%ld) is not allowed to submit mail",
+		  errstr, (long) uid);
 
     /*
      * Stop run-away process accidents by limiting the queue file size. This
@@ -280,6 +314,11 @@ int     main(int argc, char **argv)
     /*
      * Set up signal handlers and a runtime error handler so that we can
      * clean up incomplete output.
+     * 
+     * postdrop_sig() uses the in-kernel SIGINT handler address as an atomic
+     * variable to prevent nested postdrop_sig() calls. For this reason, the
+     * SIGINT handler must be configured before other signal handlers are
+     * allowed to invoke postdrop_sig().
      */
     signal(SIGPIPE, SIG_IGN);
     signal(SIGXFSZ, SIG_IGN);
@@ -328,7 +367,7 @@ int     main(int argc, char **argv)
 	if (rec_type == REC_TYPE_EOF) {		/* request cancelled */
 	    mail_stream_cleanup(dst);
 	    if (remove(postdrop_path))
-		msg_warn("uid=%ld: remove %s: %m", (long) getuid(), postdrop_path);
+		msg_warn("uid=%ld: remove %s: %m", (long) uid, postdrop_path);
 	    else if (msg_verbose)
 		msg_info("remove %s", postdrop_path);
 	    myfree(postdrop_path);
