@@ -2,11 +2,11 @@
 /* NAME
 /*	smtp 8
 /* SUMMARY
-/*	Postfix remote delivery via SMTP
+/*	Postfix SMTP client
 /* SYNOPSIS
 /*	\fBsmtp\fR [generic Postfix daemon options]
 /* DESCRIPTION
-/*	The SMTP client processes message delivery requests from
+/*	The Postfix SMTP client processes message delivery requests from
 /*	the queue manager. Each request specifies a queue file, a sender
 /*	address, a domain or host to deliver to, and recipient information.
 /*	This program expects to be run from the \fBmaster\fR(8) process
@@ -25,6 +25,15 @@
 /*	When a server is not reachable, or when mail delivery fails due
 /*	to a recoverable error condition, the SMTP client will try to
 /*	deliver the mail to an alternate host.
+/*
+/*	After a successful mail transaction, a connection may be saved
+/*	to the \fBscache\fR(8) connection cache server, so that it
+/*	may be used by any SMTP client for a subsequent transaction.
+/*
+/*	By default, connection caching is enabled temporarily for
+/*	destinations that have a high volume of mail in the active
+/*	queue. Session caching can be enabled permanently for
+/*	specific destinations.
 /* SECURITY
 /* .ad
 /* .fi
@@ -42,6 +51,7 @@
 /*	RFC 2554 (AUTH command)
 /*	RFC 2821 (SMTP protocol)
 /*	RFC 2920 (SMTP Pipelining)
+/*	RFC 3207 (STARTTLS command)
 /* DIAGNOSTICS
 /*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /*	Corrupted message files are marked so that the queue manager can
@@ -50,15 +60,22 @@
 /*	Depending on the setting of the \fBnotify_classes\fR parameter,
 /*	the postmaster is notified of bounces, protocol problems, and of
 /*	other trouble.
+/* BUGS
+/*	SMTP connection caching does not work with TLS. The necessary
+/*	support for TLS object passivation and re-activation does not
+/*	exist without closing the session, which defeats the purpose.
+/*
+/*	SMTP connection caching assumes that SASL credentials are valid for
+/*	all destinations that map onto the same IP address and TCP port.
 /* CONFIGURATION PARAMETERS
 /* .ad
 /* .fi
-/*	Changes to \fBmain.cf\fR are picked up automatically, as smtp(8)
+/*	Changes to \fBmain.cf\fR are picked up automatically, as \fBsmtp\fR(8)
 /*	processes run for only a limited amount of time. Use the command
 /*	"\fBpostfix reload\fR" to speed up a change.
 /*
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /* COMPATIBILITY CONTROLS
 /* .ad
 /* .fi
@@ -94,6 +111,21 @@
 /* .IP "\fBsmtp_skip_4xx_greeting (yes)\fR"
 /*	Skip SMTP servers that greet with a 4XX status code (go away, try
 /*	again later).
+/* .PP
+/*	Available in Postfix version 2.2 and later:
+/* .IP "\fBsmtp_discard_ehlo_keyword_address_maps (empty)\fR"
+/*	Lookup tables, indexed by the remote SMTP server address, with
+/*	case insensitive lists of EHLO keywords (pipelining, starttls,
+/*	auth, etc.) that the SMTP client will ignore in the EHLO response
+/*	from a remote SMTP server.
+/* .IP "\fBsmtp_discard_ehlo_keywords (empty)\fR"
+/*	A case insensitive list of EHLO keywords (pipelining, starttls,
+/*	auth, etc.) that the SMTP client will ignore in the EHLO response
+/*	from a remote SMTP server.
+/* .IP "\fBsmtp_generic_maps (empty)\fR"
+/*	Optional lookup tables that perform address rewriting in the
+/*	SMTP client, typically to transform a locally valid address into
+/*	a globally valid address when sending mail across the Internet.
 /* MIME PROCESSING CONTROLS
 /* .ad
 /* .fi
@@ -103,8 +135,7 @@
 /* .IP "\fBmime_boundary_length_limit (2048)\fR"
 /*	The maximal length of MIME multipart boundary strings.
 /* .IP "\fBmime_nesting_limit (100)\fR"
-/*	The maximal nesting level of multipart mail that the MIME processor
-/*	will handle.
+/*	The maximal recursion level that the MIME processor will handle.
 /* EXTERNAL CONTENT INSPECTION CONTROLS
 /* .ad
 /* .fi
@@ -123,6 +154,69 @@
 /* .IP "\fBsmtp_sasl_security_options (noplaintext, noanonymous)\fR"
 /*	What authentication mechanisms the Postfix SMTP client is allowed
 /*	to use.
+/* .PP
+/*	Available in Postfix version 2.2 and later:
+/* .IP "\fBsmtp_sasl_mechanism_filter (empty)\fR"
+/*	If non-empty, a Postfix SMTP client filter for the remote SMTP
+/*	server's list of offered SASL mechanisms.
+/* STARTTLS SUPPORT CONTROLS
+/* .ad
+/* .fi
+/*	Detailed information about STARTTLS configuration may be found
+/*	in the TLS_README document.
+/* .IP "\fBsmtp_use_tls (no)\fR"
+/*	Opportunistic mode: use TLS when a remote SMTP server announces
+/*	STARTTLS support, otherwise send the mail in the clear.
+/* .IP "\fBsmtp_enforce_tls (no)\fR"
+/*	Enforcement mode: require that remote SMTP servers use TLS
+/*	encryption, and never send mail in the clear.
+/* .IP "\fBsmtp_sasl_tls_security_options ($smtp_sasl_security_options)\fR"
+/*	The SASL authentication security options that the Postfix SMTP
+/*	client uses for TLS encrypted SMTP sessions.
+/* .IP "\fBsmtp_starttls_timeout (300s)\fR"
+/*	Time limit for Postfix SMTP client write and read operations
+/*	during TLS startup and shutdown handshake procedures.
+/* .IP "\fBsmtp_tls_CAfile (empty)\fR"
+/*	The file with the certificate of the certification authority
+/*	(CA) that issued the Postfix SMTP client certificate.
+/* .IP "\fBsmtp_tls_CApath (empty)\fR"
+/*	Directory with PEM format certificate authority certificates
+/*	that the Postfix SMTP client uses to verify a remote SMTP server
+/*	certificate.
+/* .IP "\fBsmtp_tls_cert_file (empty)\fR"
+/*	File with the Postfix SMTP client RSA certificate in PEM format.
+/* .IP "\fBsmtp_tls_cipherlist (empty)\fR"
+/*	Controls the Postfix SMTP client TLS cipher selection scheme.
+/* .IP "\fBsmtp_tls_dcert_file (empty)\fR"
+/*	File with the Postfix SMTP client DSA certificate in PEM format.
+/* .IP "\fBsmtp_tls_dkey_file ($smtp_tls_dcert_file)\fR"
+/*	File with the Postfix SMTP client DSA private key in PEM format.
+/* .IP "\fBsmtp_tls_enforce_peername (yes)\fR"
+/*	When TLS encryption is enforced, require that the remote SMTP
+/*	server hostname matches the information in the remote SMTP server
+/*	certificate.
+/* .IP "\fBsmtp_tls_key_file ($smtp_tls_cert_file)\fR"
+/*	File with the Postfix SMTP client RSA private key in PEM format.
+/* .IP "\fBsmtp_tls_loglevel (0)\fR"
+/*	Enable additional Postfix SMTP client logging of TLS activity.
+/* .IP "\fBsmtp_tls_note_starttls_offer (no)\fR"
+/*	Log the hostname of a remote SMTP server that offers STARTTLS,
+/*	when TLS is not already enabled for that server.
+/* .IP "\fBsmtp_tls_per_site (empty)\fR"
+/*	Optional lookup tables with the Postfix SMTP client TLS usage
+/*	policy by next-hop domain name and by remote SMTP server hostname.
+/* .IP "\fBsmtp_tls_scert_verifydepth (5)\fR"
+/*	The verification depth for remote SMTP server certificates.
+/* .IP "\fBsmtp_tls_session_cache_database (empty)\fR"
+/*	Name of the file containing the optional Postfix SMTP client
+/*	TLS session cache.
+/* .IP "\fBsmtp_tls_session_cache_timeout (3600s)\fR"
+/*	The expiration time of Postfix SMTP client TLS session cache
+/*	information.
+/* .IP "\fBtls_daemon_random_bytes (32)\fR"
+/*	The number of pseudo-random bytes that an \fBsmtp\fR(8) or \fBsmtpd\fR(8)
+/*	process requests from the \fBtlsmgr\fR(8) server in order to seed its
+/*	internal pseudo random number generator (PRNG).
 /* RESOURCE AND RATE CONTROLS
 /* .ad
 /* .fi
@@ -167,9 +261,23 @@
 /*	The maximal number of SMTP sessions per delivery request before
 /*	giving up or delivering to a fall-back relay host, or zero (no
 /*	limit).
-/* .IP "\fBsmtp_rset_timeout (120s)\fR"
+/* .IP "\fBsmtp_rset_timeout (20s)\fR"
 /*	The SMTP client time limit for sending the RSET command, and
 /*	for receiving the server response.
+/* .PP
+/*	Available in Postfix version 2.2 and later:
+/* .IP "\fBsmtp_connection_cache_destinations (empty)\fR"
+/*	Permanently enable SMTP connection caching for the specified
+/*	destinations.
+/* .IP "\fBsmtp_connection_cache_on_demand (yes)\fR"
+/*	Temporarily enable SMTP connection caching while a destination
+/*	has a high volume of mail in the active queue.
+/* .IP "\fBsmtp_connection_cache_reuse_limit (10)\fR"
+/*	When SMTP connection caching is enabled, the number of times that
+/*	an SMTP session is reused before it is closed.
+/* .IP "\fBsmtp_connection_cache_time_limit (2s)\fR"
+/*	When SMTP connection caching is enabled, the amount of time that
+/*	an unused SMTP client socket is kept open before it is closed.
 /* TROUBLE SHOOTING CONTROLS
 /* .ad
 /* .fi
@@ -204,8 +312,11 @@
 /*	Optional list of relay hosts for SMTP destinations that can't be
 /*	found or that are unreachable.
 /* .IP "\fBinet_interfaces (all)\fR"
-/*	The network interface addresses that this mail system receives mail
-/*	on.
+/*	The network interface addresses that this mail system receives
+/*	mail on.
+/* .IP "\fBinet_protocols (ipv4)\fR"
+/*	The Internet protocols Postfix will attempt to use when making
+/*	or accepting connections.
 /* .IP "\fBipc_timeout (3600s)\fR"
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
@@ -224,7 +335,10 @@
 /*	on by way of a proxy or network address translation unit.
 /* .IP "\fBsmtp_bind_address (empty)\fR"
 /*	An optional numerical network address that the SMTP client should
-/*	bind to when making a connection.
+/*	bind to when making an IPv4 connection.
+/* .IP "\fBsmtp_bind_address6 (empty)\fR"
+/*	An optional numerical network address that the SMTP client should
+/*	bind to when making an IPv6 connection.
 /* .IP "\fBsmtp_helo_name ($myhostname)\fR"
 /*	The hostname to send in the SMTP EHLO or HELO command.
 /* .IP "\fBsmtp_host_lookup (dns)\fR"
@@ -240,8 +354,11 @@
 /* SEE ALSO
 /*	qmgr(8), queue manager
 /*	bounce(8), delivery status reports
+/*	scache(8), connection cache server
 /*	postconf(5), configuration parameters
+/*	master(5), generic daemon options
 /*	master(8), process manager
+/*	tlsmgr(8), TLS session and PRNG management
 /*	syslogd(8), system logging
 /* README FILES
 /* .ad
@@ -251,6 +368,7 @@
 /* .na
 /* .nf
 /*	SASL_README, Postfix SASL howto
+/*	TLS_README, Postfix STARTTLS howto
 /* LICENSE
 /* .ad
 /* .fi
@@ -260,6 +378,25 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Command pipelining in cooperation with:
+/*	Jon Ribbens
+/*	Oaktree Internet Solutions Ltd.,
+/*	Internet House,
+/*	Canal Basin,
+/*	Coventry,
+/*	CV1 4LY, United Kingdom.
+/*
+/*	Connection caching in cooperation with:
+/*	Victor Duchovni
+/*	Morgan Stanley
+/*
+/*	TLS support originally by:
+/*	Lutz Jaenicke
+/*	BTU Cottbus
+/*	Allgemeine Elektrotechnik
+/*	Universitaetsplatz 3-4
+/*	D-03044 Cottbus, Germany
 /*--*/
 
 /* System library. */
@@ -284,6 +421,10 @@
 #include <mail_conf.h>
 #include <debug_peer.h>
 #include <flush_clnt.h>
+#include <scache.h>
+#include <string_list.h>
+#include <maps.h>
+#include <ext_prop.h>
 
 /* Single server skeleton. */
 
@@ -321,7 +462,9 @@ int     var_smtp_never_ehlo;
 char   *var_smtp_sasl_opts;
 char   *var_smtp_sasl_passwd;
 bool    var_smtp_sasl_enable;
+char   *var_smtp_sasl_mechs;
 char   *var_smtp_bind_addr;
+char   *var_smtp_bind_addr6;
 bool    var_smtp_rand_addr;
 int     var_smtp_pix_thresh;
 int     var_smtp_pix_delay;
@@ -333,6 +476,27 @@ bool    var_smtp_defer_mxaddr;
 bool    var_smtp_send_xforward;
 int     var_smtp_mxaddr_limit;
 int     var_smtp_mxsess_limit;
+int     var_smtp_cache_conn;
+int     var_smtp_reuse_limit;
+char   *var_smtp_cache_dest;
+char   *var_scache_service;
+bool    var_smtp_cache_demand;
+char   *var_smtp_ehlo_dis_words;
+char   *var_smtp_ehlo_dis_maps;
+
+bool    var_smtp_use_tls;
+bool    var_smtp_enforce_tls;
+char   *var_smtp_tls_per_site;
+#ifdef USE_TLS
+int     var_smtp_starttls_tmout;
+char   *var_smtp_sasl_tls_opts;
+bool    var_smtp_tls_enforce_peername;
+int     var_smtp_tls_scert_vd;
+bool    var_smtp_tls_note_starttls_offer;
+#endif
+
+char   *var_smtp_generic_maps;
+char   *var_prop_extension;
 
  /*
   * Global variables. smtp_errno is set by the address lookup routines and by
@@ -340,10 +504,24 @@ int     var_smtp_mxsess_limit;
   */
 int     smtp_errno;
 int     smtp_host_lookup_mask;
+STRING_LIST *smtp_cache_dest;
+SCACHE *smtp_scache;
+MAPS   *smtp_ehlo_dis_maps;
+MAPS   *smtp_generic_maps;
+int     smtp_ext_prop_mask;
+
+#ifdef USE_TLS
+
+ /*
+  * OpenSSL client state.
+  */
+SSL_CTX *smtp_tls_ctx;
+
+#endif
 
 /* deliver_message - deliver message with extreme prejudice */
 
-static int deliver_message(DELIVER_REQUEST *request)
+static int deliver_message(const char *service, DELIVER_REQUEST *request)
 {
     SMTP_STATE *state;
     int     result;
@@ -369,6 +547,7 @@ static int deliver_message(DELIVER_REQUEST *request)
     state = smtp_state_alloc();
     state->request = request;
     state->src = request->fp;
+    state->service = service;
     SMTP_RCPT_INIT(state);
 
     /*
@@ -382,7 +561,6 @@ static int deliver_message(DELIVER_REQUEST *request)
     /*
      * Clean up.
      */
-    smtp_chat_reset(state);
     smtp_state_free(state);
 
     return (result);
@@ -390,7 +568,7 @@ static int deliver_message(DELIVER_REQUEST *request)
 
 /* smtp_service - perform service for client */
 
-static void smtp_service(VSTREAM *client_stream, char *unused_service, char **argv)
+static void smtp_service(VSTREAM *client_stream, char *service, char **argv)
 {
     DELIVER_REQUEST *request;
     int     status;
@@ -410,14 +588,14 @@ static void smtp_service(VSTREAM *client_stream, char *unused_service, char **ar
      * the common code in single_server.c.
      */
     if ((request = deliver_request_read(client_stream)) != 0) {
-	status = deliver_message(request);
+	status = deliver_message(service, request);
 	deliver_request_done(client_stream, request, status);
     }
 }
 
-/* pre_init - pre-jail initialization */
+/* post_init - post-jail initialization */
 
-static void pre_init(char *unused_name, char **unused_argv)
+static void post_init(char *unused_name, char **unused_argv)
 {
     static NAME_MASK lookup_masks[] = {
 	SMTP_HOST_LOOKUP_DNS, SMTP_HOST_FLAG_DNS,
@@ -425,11 +603,6 @@ static void pre_init(char *unused_name, char **unused_argv)
 	0,
     };
 
-    /*
-     * Turn on per-peer debugging.
-     */
-    debug_peer_init();
-
     /*
      * Select hostname lookup mechanisms.
      */
@@ -443,6 +616,29 @@ static void pre_init(char *unused_name, char **unused_argv)
 		 str_name_mask(VAR_SMTP_HOST_LOOKUP, lookup_masks,
 			       smtp_host_lookup_mask));
 
+    /*
+     * Session cache instance.
+     */
+    if (*var_smtp_cache_dest || var_smtp_cache_demand)
+#if 0
+	smtp_scache = scache_multi_create();
+#else
+	smtp_scache = scache_clnt_create(var_scache_service,
+					 var_ipc_idle_limit,
+					 var_ipc_ttl_limit);
+#endif
+}
+
+/* pre_init - pre-jail initialization */
+
+static void pre_init(char *unused_name, char **unused_argv)
+{
+
+    /*
+     * Turn on per-peer debugging.
+     */
+    debug_peer_init();
+
     /*
      * SASL initialization.
      */
@@ -454,10 +650,47 @@ static void pre_init(char *unused_name, char **unused_argv)
 		 VAR_SMTP_SASL_ENABLE);
 #endif
 
+    /*
+     * Initialize the TLS data before entering the chroot jail
+     */
+    if (var_smtp_use_tls || var_smtp_enforce_tls || var_smtp_tls_per_site[0]) {
+#ifdef USE_TLS
+	smtp_tls_ctx = tls_client_init(var_smtp_tls_scert_vd);
+	smtp_tls_list_init();
+#else
+	msg_warn("TLS has been selected, but TLS support is not compiled in");
+#endif
+    }
+
     /*
      * Flush client.
      */
     flush_init();
+
+    /*
+     * Session cache domain list.
+     */
+    if (*var_smtp_cache_dest)
+	smtp_cache_dest = string_list_init(MATCH_FLAG_NONE, var_smtp_cache_dest);
+
+    /*
+     * EHLO keyword filter.
+     */
+    if (*var_smtp_ehlo_dis_maps)
+	smtp_ehlo_dis_maps = maps_create(VAR_SMTPD_EHLO_DIS_MAPS,
+					 var_smtp_ehlo_dis_maps,
+					 DICT_FLAG_LOCK);
+
+    /*
+     * Generic maps.
+     */
+    if (*var_prop_extension)
+	smtp_ext_prop_mask =
+	    ext_prop_mask(VAR_PROP_EXTENSION, var_prop_extension);
+    if (*var_smtp_generic_maps)
+	smtp_generic_maps =
+	    maps_create(VAR_SMTP_GENERIC_MAPS, var_smtp_generic_maps,
+			DICT_FLAG_LOCK);
 }
 
 /* pre_accept - see if tables have changed */
@@ -493,9 +726,21 @@ int     main(int argc, char **argv)
 	VAR_ERROR_RCPT, DEF_ERROR_RCPT, &var_error_rcpt, 1, 0,
 	VAR_SMTP_SASL_PASSWD, DEF_SMTP_SASL_PASSWD, &var_smtp_sasl_passwd, 0, 0,
 	VAR_SMTP_SASL_OPTS, DEF_SMTP_SASL_OPTS, &var_smtp_sasl_opts, 0, 0,
+#ifdef USE_TLS
+	VAR_SMTP_SASL_TLS_OPTS, DEF_SMTP_SASL_TLS_OPTS, &var_smtp_sasl_tls_opts, 0, 0,
+#endif
+	VAR_SMTP_SASL_MECHS, DEF_SMTP_SASL_MECHS, &var_smtp_sasl_mechs, 0, 0,
 	VAR_SMTP_BIND_ADDR, DEF_SMTP_BIND_ADDR, &var_smtp_bind_addr, 0, 0,
+	VAR_SMTP_BIND_ADDR6, DEF_SMTP_BIND_ADDR6, &var_smtp_bind_addr6, 0, 0,
 	VAR_SMTP_HELO_NAME, DEF_SMTP_HELO_NAME, &var_smtp_helo_name, 1, 0,
 	VAR_SMTP_HOST_LOOKUP, DEF_SMTP_HOST_LOOKUP, &var_smtp_host_lookup, 1, 0,
+	VAR_SMTP_CACHE_DEST, DEF_SMTP_CACHE_DEST, &var_smtp_cache_dest, 0, 0,
+	VAR_SCACHE_SERVICE, DEF_SCACHE_SERVICE, &var_scache_service, 1, 0,
+	VAR_SMTP_EHLO_DIS_WORDS, DEF_SMTP_EHLO_DIS_WORDS, &var_smtp_ehlo_dis_words, 0, 0,
+	VAR_SMTP_EHLO_DIS_MAPS, DEF_SMTP_EHLO_DIS_MAPS, &var_smtp_ehlo_dis_maps, 0, 0,
+	VAR_SMTP_TLS_PER_SITE, DEF_SMTP_TLS_PER_SITE, &var_smtp_tls_per_site, 0, 0,
+	VAR_PROP_EXTENSION, DEF_PROP_EXTENSION, &var_prop_extension, 0, 0,
+	VAR_SMTP_GENERIC_MAPS, DEF_SMTP_GENERIC_MAPS, &var_smtp_generic_maps, 0, 0,
 	0,
     };
     static CONFIG_TIME_TABLE time_table[] = {
@@ -511,12 +756,20 @@ int     main(int argc, char **argv)
 	VAR_SMTP_QUIT_TMOUT, DEF_SMTP_QUIT_TMOUT, &var_smtp_quit_tmout, 1, 0,
 	VAR_SMTP_PIX_THRESH, DEF_SMTP_PIX_THRESH, &var_smtp_pix_thresh, 0, 0,
 	VAR_SMTP_PIX_DELAY, DEF_SMTP_PIX_DELAY, &var_smtp_pix_delay, 1, 0,
+	VAR_SMTP_CACHE_CONN, DEF_SMTP_CACHE_CONN, &var_smtp_cache_conn, 1, 0,
+#ifdef USE_TLS
+	VAR_SMTP_STARTTLS_TMOUT, DEF_SMTP_STARTTLS_TMOUT, &var_smtp_starttls_tmout, 1, 0,
+#endif
 	0,
     };
     static CONFIG_INT_TABLE int_table[] = {
 	VAR_SMTP_LINE_LIMIT, DEF_SMTP_LINE_LIMIT, &var_smtp_line_limit, 0, 0,
 	VAR_SMTP_MXADDR_LIMIT, DEF_SMTP_MXADDR_LIMIT, &var_smtp_mxaddr_limit, 0, 0,
 	VAR_SMTP_MXSESS_LIMIT, DEF_SMTP_MXSESS_LIMIT, &var_smtp_mxsess_limit, 0, 0,
+	VAR_SMTP_REUSE_LIMIT, DEF_SMTP_REUSE_LIMIT, &var_smtp_reuse_limit, 1, 0,
+#ifdef USE_TLS
+	VAR_SMTP_TLS_SCERT_VD, DEF_SMTP_TLS_SCERT_VD, &var_smtp_tls_scert_vd, 0, 0,
+#endif
 	0,
     };
     static CONFIG_BOOL_TABLE bool_table[] = {
@@ -530,6 +783,14 @@ int     main(int argc, char **argv)
 	VAR_SMTP_QUOTE_821_ENV, DEF_SMTP_QUOTE_821_ENV, &var_smtp_quote_821_env,
 	VAR_SMTP_DEFER_MXADDR, DEF_SMTP_DEFER_MXADDR, &var_smtp_defer_mxaddr,
 	VAR_SMTP_SEND_XFORWARD, DEF_SMTP_SEND_XFORWARD, &var_smtp_send_xforward,
+	VAR_SMTP_CACHE_DEMAND, DEF_SMTP_CACHE_DEMAND, &var_smtp_cache_demand,
+	VAR_SMTP_USE_TLS, DEF_SMTP_USE_TLS, &var_smtp_use_tls,
+	VAR_SMTP_ENFORCE_TLS, DEF_SMTP_ENFORCE_TLS, &var_smtp_enforce_tls,
+#ifdef USE_TLS
+	VAR_SMTP_TLS_ENFORCE_PN, DEF_SMTP_TLS_ENFORCE_PN, &var_smtp_tls_enforce_peername,
+	VAR_SMTP_TLS_NOTEOFFER, DEF_SMTP_TLS_NOTEOFFER, &var_smtp_tls_note_starttls_offer,
+#endif
+
 	0,
     };
 
@@ -539,6 +800,7 @@ int     main(int argc, char **argv)
 		       MAIL_SERVER_STR_TABLE, str_table,
 		       MAIL_SERVER_BOOL_TABLE, bool_table,
 		       MAIL_SERVER_PRE_INIT, pre_init,
+		       MAIL_SERVER_POST_INIT, post_init,
 		       MAIL_SERVER_PRE_ACCEPT, pre_accept,
 		       MAIL_SERVER_EXIT, pre_exit,
 		       0);
