@@ -1,6 +1,6 @@
 /*++
 /* NAME
-/*	smtp-sink 8
+/*	smtp-sink 1
 /* SUMMARY
 /*	multi-threaded SMTP/LMTP test server
 /* SYNOPSIS
@@ -10,32 +10,53 @@
 /*
 /*	\fBsmtp-sink\fR [\fIoptions\fR] \fBunix:\fR\fIpathname\fR \fIbacklog\fR
 /* DESCRIPTION
-/*	\fIsmtp-sink\fR listens on the named host (or address) and port.
+/*	\fBsmtp-sink\fR listens on the named host (or address) and port.
 /*	It takes SMTP messages from the network and throws them away.
-/*	The purpose is to measure SMTP client performance, not protocol
+/*	The purpose is to measure client performance, not protocol
 /*	compliance.
-/*	Connections can be accepted on IPV4 endpoints or UNIX-domain sockets.
-/*	IPV4 is the default.
-/*	This program is the complement of the \fIsmtp-source\fR program.
+/*
+/*	Connections can be accepted on IPv4 or IPv6 endpoints, or on
+/*	UNIX-domain sockets.
+/*	IPv4 and IPv6 are the default.
+/*	This program is the complement of the \fBsmtp-source\fR(1) program.
 /*
 /*	Arguments:
+/* .IP \fB-4\fR
+/*	Support IPv4 only. This option has no effect when
+/*	Postfix is built without IPv6 support.
+/* .IP \fB-6\fR
+/*	Support IPv6 only. This option is not available when
+/*	Postfix is built without IPv6 support.
+/* .IP \fB-a\fR
+/*	Do not announce SASL authentication support.
 /* .IP \fB-c\fR
 /*	Display a running counter that is updated whenever an SMTP
 /*	QUIT command is executed.
+/* .IP \fB-C\fR
+/*	Disable XCLIENT support.
 /* .IP \fB-e\fR
-/*	Disable ESMTP support.
-/* .IP \fB-h\fI hostname\fR
+/*	Do not announce ESMTP support.
+/* .IP "\fB-f \fIcommand,command,...\fR"
+/*	Reject the specified commands with a hard (5xx) error code.
+/* .IP \fB-F\fR
+/*	Disable XFORWARD support.
+/* .IP "\fB-h\fI hostname\fR"
 /*	Use \fIhostname\fR in the SMTP greeting, in the HELO response,
 /*	and in the EHLO response. The default hostname is "smtp-sink".
 /* .IP \fB-L\fR
-/*	Enable LMTP rather than SMTP.
+/*	Enable LMTP instead of SMTP.
 /* .IP "\fB-n \fIcount\fR"
 /*	Terminate after \fIcount\fR sessions. This is for testing purposes.
 /* .IP \fB-p\fR
-/*	Disable ESMTP command pipelining.
+/*	Do not announce support for ESMTP command pipelining.
 /* .IP \fB-P\fR
 /*	Change the server greeting so that it appears to come through
-/*	a CISCO PIX system.
+/*	a CISCO PIX system. Implies \fB-e\fR.
+/* .IP "\fB-q \fIcommand,command,...\fR"
+/*	Disconnect (without replying) after receiving one of the
+/*	specified commands.
+/* .IP "\fB-r \fIcommand,command,...\fR"
+/*	Reject the specified commands with a soft (4xx) error code.
 /* .IP "\fB-s \fIcommand,command,...\fR"
 /*	Log the named commands to syslogd.
 /*	Examples of commands that can be logged are HELO, EHLO, LHLO, MAIL,
@@ -47,7 +68,7 @@
 /* .IP "\fB-w \fIdelay\fR"
 /*	Wait \fIdelay\fR seconds before responding to a DATA command.
 /* .IP \fB-8\fR
-/*	Disable 8BITMIME support.
+/*	Do not announce 8BITMIME support.
 /* .IP [\fBinet:\fR][\fIhost\fR]:\fIport\fR
 /*	Listen on network interface \fIhost\fR (default: any interface)
 /*	TCP port \fIport\fR. Both \fIhost\fR and \fIport\fR may be
@@ -56,9 +77,9 @@
 /*	Listen on the UNIX-domain socket at \fIpathname\fR.
 /* .IP \fIbacklog\fR
 /*	The maximum length the queue of pending connections,
-/*	as defined by the listen(2) call.
+/*	as defined by the \fBlisten\fR(2) system call.
 /* SEE ALSO
-/*	smtp-source, SMTP/LMTP test message generator
+/*	smtp-source(1), SMTP/LMTP message generator
 /* LICENSE
 /* .ad
 /* .fi
@@ -99,6 +120,7 @@
 #include <msg_vstream.h>
 #include <stringops.h>
 #include <sane_accept.h>
+#include <inet_proto.h>
 
 /* Global library. */
 
@@ -110,7 +132,7 @@ typedef struct SINK_STATE {
     VSTREAM *stream;
     VSTRING *buffer;
     int     data_state;
-    int     (*read) (struct SINK_STATE *);
+    int     (*read_fn) (struct SINK_STATE *);
     int     rcpts;
 } SINK_STATE;
 
@@ -136,6 +158,9 @@ static int fixed_delay;
 static int disable_esmtp;
 static int enable_lmtp;
 static int pretend_pix;
+static int disable_saslauth;
+static int disable_xclient;
+static int disable_xforward;
 
 /* ehlo_response - respond to EHLO command */
 
@@ -146,7 +171,14 @@ static void ehlo_response(SINK_STATE *state)
 	smtp_printf(state->stream, "250-PIPELINING");
     if (!disable_8bitmime)
 	smtp_printf(state->stream, "250-8BITMIME");
+    if (!disable_saslauth)
+	smtp_printf(state->stream, "250-AUTH PLAIN LOGIN");
+    if (!disable_xclient)
+	smtp_printf(state->stream, "250-XCLIENT NAME HELO");
+    if (!disable_xforward)
+	smtp_printf(state->stream, "250-XFORWARD NAME ADDR PROTO HELO");
     smtp_printf(state->stream, "250 ");
+    smtp_flush(state->stream);
 }
 
 /* helo_response - respond to HELO command */
@@ -154,6 +186,7 @@ static void ehlo_response(SINK_STATE *state)
 static void helo_response(SINK_STATE *state)
 {
     smtp_printf(state->stream, "250 %s", var_myhostname);
+    smtp_flush(state->stream);
 }
 
 /* ok_response - send 250 OK */
@@ -161,6 +194,7 @@ static void helo_response(SINK_STATE *state)
 static void ok_response(SINK_STATE *state)
 {
     smtp_printf(state->stream, "250 Ok");
+    smtp_flush(state->stream);
 }
 
 /* mail_response - reset recipient count, send 250 OK */
@@ -185,7 +219,8 @@ static void data_response(SINK_STATE *state)
 {
     state->data_state = ST_CR_LF;
     smtp_printf(state->stream, "354 End data with <CR><LF>.<CR><LF>");
-    state->read = data_read;
+    smtp_flush(state->stream);
+    state->read_fn = data_read;
 }
 
 /* data_event - delayed response to DATA command */
@@ -203,7 +238,7 @@ static void dot_response(SINK_STATE *state)
 {
     if (enable_lmtp) {
 	while (state->rcpts-- > 0)	/* XXX this could block */
-	    ok_response(state);
+	    ok_response(state);		/* XXX this flushes too often */
     } else {
 	ok_response(state);
     }
@@ -214,6 +249,7 @@ static void dot_response(SINK_STATE *state)
 static void quit_response(SINK_STATE *state)
 {
     smtp_printf(state->stream, "221 Bye");
+    smtp_flush(state->stream);
     if (count) {
 	counter++;
 	vstream_printf("%d\r", counter);
@@ -266,7 +302,7 @@ static int data_read(SINK_STATE *state)
 	    if (msg_verbose)
 		msg_info(".");
 	    dot_response(state);
-	    state->read = command_read;
+	    state->read_fn = command_read;
 	    state->data_state = ST_ANY;
 	    break;
 	}
@@ -293,11 +329,17 @@ typedef struct SINK_COMMAND {
 
 #define FLAG_ENABLE	(1<<0)		/* command is enabled */
 #define FLAG_SYSLOG	(1<<1)		/* log the command */
+#define FLAG_HARD_ERR	(1<<2)		/* report hard error */
+#define FLAG_SOFT_ERR	(1<<3)		/* report soft error */
+#define FLAG_DISCONNECT	(1<<4)		/* disconnect */
 
 static SINK_COMMAND command_table[] = {
     "helo", helo_response, 0,
     "ehlo", ehlo_response, 0,
     "lhlo", ehlo_response, 0,
+    "xclient", ok_response, FLAG_ENABLE,
+    "xforward", ok_response, FLAG_ENABLE,
+    "auth", ok_response, FLAG_ENABLE,
     "mail", mail_response, FLAG_ENABLE,
     "rcpt", rcpt_response, FLAG_ENABLE,
     "data", data_response, FLAG_ENABLE,
@@ -308,6 +350,20 @@ static SINK_COMMAND command_table[] = {
     0,
 };
 
+/* reset_cmd_flags - reset per-command command flags */
+
+static void reset_cmd_flags(const char *cmd, int flags)
+{
+    SINK_COMMAND *cmdp;
+
+    for (cmdp = command_table; cmdp->name != 0; cmdp++)
+	if (strcasecmp(cmd, cmdp->name) == 0)
+	    break;
+    if (cmdp->name == 0)
+	msg_fatal("unknown command: %s", cmd);
+    cmdp->flags &= ~flags;
+}
+
 /* set_cmd_flags - set per-command command flags */
 
 static void set_cmd_flags(const char *cmd, int flags)
@@ -417,17 +473,31 @@ static int command_read(SINK_STATE *state)
      * Got a complete command line. Parse it.
      */
     ptr = vstring_str(state->buffer);
+    if (msg_verbose)
+	msg_info("%s", ptr);
     if ((command = mystrtok(&ptr, " \t")) == 0) {
 	smtp_printf(state->stream, "500 Error: unknown command");
+	smtp_flush(state->stream);
 	return (0);
     }
-    if (msg_verbose)
-	msg_info("%s", command);
     for (cmdp = command_table; cmdp->name != 0; cmdp++)
 	if (strcasecmp(command, cmdp->name) == 0)
 	    break;
     if (cmdp->name == 0 || (cmdp->flags & FLAG_ENABLE) == 0) {
 	smtp_printf(state->stream, "500 Error: unknown command");
+	smtp_flush(state->stream);
+	return (0);
+    }
+    if (cmdp->flags & FLAG_DISCONNECT)
+	return (-1);
+    if (cmdp->flags & FLAG_HARD_ERR) {
+	smtp_printf(state->stream, "500 Error: command failed");
+	smtp_flush(state->stream);
+	return (0);
+    }
+    if (cmdp->flags & FLAG_SOFT_ERR) {
+	smtp_printf(state->stream, "450 Error: command failed");
+	smtp_flush(state->stream);
 	return (0);
     }
     /* We use raw syslog. Sanitize data content and length. */
@@ -465,7 +535,7 @@ static void read_event(int unused_event, char *context)
 	    return;
 
 	case 0:
-	    if (state->read(state) < 0) {
+	    if (state->read_fn(state) < 0) {
 		if (msg_verbose)
 		    msg_info("disconnect");
 		disconnect(state);
@@ -514,7 +584,7 @@ static void connect_event(int unused_event, char *context)
 	state = (SINK_STATE *) mymalloc(sizeof(*state));
 	state->stream = vstream_fdopen(fd, O_RDWR);
 	state->buffer = vstring_alloc(1024);
-	state->read = command_read;
+	state->read_fn = command_read;
 	state->data_state = ST_ANY;
 	smtp_timeout_setup(state->stream, var_tmout);
 	if (pretend_pix)
@@ -523,6 +593,7 @@ static void connect_event(int unused_event, char *context)
 	    smtp_printf(state->stream, "220 %s", var_myhostname);
 	else
 	    smtp_printf(state->stream, "220 %s ESMTP", var_myhostname);
+	smtp_flush(state->stream);
 	event_enable_read(fd, read_event, (char *) state);
     }
 }
@@ -531,7 +602,7 @@ static void connect_event(int unused_event, char *context)
 
 static void usage(char *myname)
 {
-    msg_fatal("usage: %s [-ceLpPv8] [-h hostname] [-n count] [-s commands] [-w delay] [host]:port backlog", myname);
+    msg_fatal("usage: %s [-acCeFLpPv8] [-f commands] [-h hostname] [-n count] [-q commands] [-r commands] [-s commands] [-w delay] [host]:port backlog", myname);
 }
 
 int     main(int argc, char **argv)
@@ -539,6 +610,8 @@ int     main(int argc, char **argv)
     int     sock;
     int     backlog;
     int     ch;
+    const char *protocols = INET_PROTO_NAME_ALL;
+    INET_PROTO_INFO *proto_info;
 
     /*
      * Initialize diagnostics.
@@ -548,14 +621,34 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "ceh:Ln:pPs:vw:8")) > 0) {
+    while ((ch = GETOPT(argc, argv, "46acCef:Fh:Ln:pPq:r:s:vw:8")) > 0) {
 	switch (ch) {
+	case '4':
+	    protocols = INET_PROTO_NAME_IPV4;
+	    break;
+	case '6':
+	    protocols = INET_PROTO_NAME_IPV6;
+	    break;
+	case 'a':
+	    disable_saslauth = 1;
+	    break;
 	case 'c':
 	    count++;
 	    break;
+	case 'C':
+	    disable_xclient = 1;
+	    reset_cmd_flags("xclient", FLAG_ENABLE);
+	    break;
 	case 'e':
 	    disable_esmtp = 1;
 	    break;
+	case 'f':
+	    set_cmds_flags(optarg, FLAG_HARD_ERR);
+	    break;
+	case 'F':
+	    disable_xforward = 1;
+	    reset_cmd_flags("xforward", FLAG_ENABLE);
+	    break;
 	case 'h':
 	    var_myhostname = optarg;
 	    break;
@@ -563,13 +656,21 @@ int     main(int argc, char **argv)
 	    enable_lmtp = 1;
 	    break;
 	case 'n':
-	    max_count = atoi(optarg);
+	    if ((max_count = atoi(optarg)) <= 0)
+		msg_fatal("bad count: %s", optarg);
 	    break;
 	case 'p':
 	    disable_pipelining = 1;
 	    break;
 	case 'P':
 	    pretend_pix = 1;
+	    disable_esmtp = 1;
+	    break;
+	case 'q':
+	    set_cmds_flags(optarg, FLAG_DISCONNECT);
+	    break;
+	case 'r':
+	    set_cmds_flags(optarg, FLAG_SOFT_ERR);
 	    break;
 	case 's':
 	    openlog(basename(argv[0]), LOG_PID, LOG_MAIL);
@@ -602,6 +703,7 @@ int     main(int argc, char **argv)
     set_cmds_flags(enable_lmtp ? "lhlo" :
 		   disable_esmtp ? "helo" :
 		   "helo, ehlo", FLAG_ENABLE);
+    proto_info = inet_proto_init("protocols", protocols);
     if (strncmp(argv[optind], "unix:", 5) == 0) {
 	sock = unix_listen(argv[optind] + 5, backlog, BLOCKING);
     } else {
