@@ -26,7 +26,7 @@
 /*	DICT	*dict;
 /*	const char *key;
 /*
-/*	char	*dict_del(dict, key)
+/*	int	dict_del(dict, key)
 /*	DICT	*dict;
 /*	const char *key;
 /*
@@ -42,6 +42,8 @@
 /*	dict_open_register(type, open)
 /*	char	*type;
 /*	DICT	*(*open) (const char *, int, int);
+/*
+/*	ARGV	*dict_mapnames()
 /* DESCRIPTION
 /*	This module implements a low-level interface to multiple
 /*	physical dictionary types.
@@ -82,13 +84,18 @@
 /*	With file-based maps, flush I/O buffers to file after each update.
 /*	Thus feature is not supported with some file-based dictionaries.
 /* .IP DICT_FLAG_NO_REGSUB
-/*      Disallow regular expression substitution from left-hand side data 
+/*      Disallow regular expression substitution from left-hand side data
 /*	into the right-hand side.
 /* .IP DICT_FLAG_NO_PROXY
 /*	Disallow access through the \fBproxymap\fR service.
+/* .IP DICT_FLAG_NO_UNAUTH
+/*	Disallow network lookup mechanisms that lack any form of
+/*	authentication (example: tcp_table; even NIS can be secured
+/*	to some extent by requiring that the server binds to a
+/*	privileged port).
 /* .IP DICT_FLAG_PARANOID
-/*	A combination of all the paranoia flags: DICT_FLAG_NO_REGSUB
-/*	and DICT_FLAG_NO_PROXY.
+/*	A combination of all the paranoia flags: DICT_FLAG_NO_REGSUB,
+/*	DICT_FLAG_NO_PROXY and DICT_FLAG_NO_UNAUTH.
 /* .PP
 /*	Specify DICT_FLAG_NONE for no special processing.
 /*
@@ -124,7 +131,7 @@
 /*	dictionary.
 /*
 /*	dict_del() removes a dictionary entry, and returns non-zero
-/*	in case of problems.
+/*	in case of success.
 /*
 /*	dict_seq() iterates over all members in the named dictionary.
 /*	func is define DICT_SEQ_FUN_FIRST (select first member) or
@@ -135,6 +142,9 @@
 /*	associated data structures.
 /*
 /*	dict_open_register() adds support for a new dictionary type.
+/*
+/*	dict_mapnames() returns a sorted list with the names of all available
+/*	dictionary types.
 /* DIAGNOSTICS
 /*	Fatal error: open error, unsupported dictionary type, attempt to
 /*	update non-writable dictionary.
@@ -153,6 +163,7 @@
 
 #include <sys_defs.h>
 #include <string.h>
+#include <stdlib.h>
 
 #ifdef STRCASECMP_IN_STRINGS_H
 #include <strings.h>
@@ -164,9 +175,11 @@
 #include <mymalloc.h>
 #include <msg.h>
 #include <dict.h>
+#include <dict_cdb.h>
 #include <dict_env.h>
 #include <dict_unix.h>
 #include <dict_tcp.h>
+#include <dict_sdbm.h>
 #include <dict_dbm.h>
 #include <dict_db.h>
 #include <dict_nis.h>
@@ -189,11 +202,17 @@ typedef struct {
 } DICT_OPEN_INFO;
 
 static DICT_OPEN_INFO dict_open_info[] = {
+#ifdef HAS_CDB
+    DICT_TYPE_CDB, dict_cdb_open,
+#endif
     DICT_TYPE_ENVIRON, dict_env_open,
     DICT_TYPE_UNIX, dict_unix_open,
 #ifdef SNAPSHOT
     DICT_TYPE_TCP, dict_tcp_open,
 #endif
+#ifdef HAS_SDBM
+    DICT_TYPE_SDBM, dict_sdbm_open,
+#endif
 #ifdef HAS_DBM
     DICT_TYPE_DBM, dict_dbm_open,
 #endif
@@ -247,7 +266,7 @@ DICT   *dict_open(const char *dict_spec, int open_flags, int dict_flags)
     DICT   *dict;
 
     if ((dict_name = split_at(saved_dict_spec, ':')) == 0)
-	msg_fatal("open dictionary: need \"type:name\" form instead of: \"%s\"",
+	msg_fatal("open dictionary: expecting \"type:name\" form instead of \"%s\"",
 		  dict_spec);
 
     dict = dict_open3(saved_dict_spec, dict_name, open_flags, dict_flags);
@@ -265,6 +284,9 @@ DICT   *dict_open3(const char *dict_type, const char *dict_name,
     DICT_OPEN_INFO *dp;
     DICT   *dict;
 
+    if (*dict_type == 0 || *dict_name == 0)
+	msg_fatal("open dictionary: expecting \"type:name\" form instead of \"%s:%s\"",
+		  dict_type, dict_name);
     if (dict_open_hash == 0)
 	dict_open_init();
     if ((dp = (DICT_OPEN_INFO *) htable_find(dict_open_hash, dict_type)) == 0)
@@ -294,6 +316,13 @@ void    dict_open_register(const char *type,
     htable_enter(dict_open_hash, dp->type, (char *) dp);
 }
 
+/* dict_sort_alpha_cpp - qsort() callback */
+
+static int dict_sort_alpha_cpp(const void *a, const void *b)
+{
+    return (strcmp(((char **) a)[0], ((char **) b)[0]));
+}
+
 /* dict_mapnames - return an ARGV of available map_names */
 
 ARGV   *dict_mapnames()
@@ -310,6 +339,8 @@ ARGV   *dict_mapnames()
 	dp = (DICT_OPEN_INFO *) ht[0]->value;
 	argv_add(mapnames, dp->type, ARGV_END);
     }
+    qsort((void *) mapnames->argv, mapnames->argc, sizeof(mapnames->argv[0]),
+	  dict_sort_alpha_cpp);
     myfree((char *) ht_info);
     argv_terminate(mapnames);
     return mapnames;
@@ -352,7 +383,7 @@ int     main(int argc, char **argv)
     int     open_flags;
     char   *bufp;
     char   *cmd;
-    char   *key;
+    const char *key;
     const char *value;
     int     ch;
 
@@ -380,18 +411,24 @@ int     main(int argc, char **argv)
     else
 	msg_fatal("unknown access mode: %s", argv[2]);
     dict_name = argv[optind];
-    dict = dict_open(dict_name, open_flags, DICT_FLAG_LOCK);
+    dict = dict_open(dict_name, open_flags, DICT_FLAG_LOCK | DICT_FLAG_DUP_REPLACE);
     dict_register(dict_name, dict);
     while (vstring_fgets_nonl(inbuf, VSTREAM_IN)) {
 	bufp = vstring_str(inbuf);
-	if ((cmd = mystrtok(&bufp, " ")) == 0 || *bufp == 0) {
-	    vstream_printf("usage: del key|get key|put key=value\n");
+	if (!isatty(0)) {
+	    vstream_printf("> %s\n", bufp);
+	    vstream_fflush(VSTREAM_OUT);
+	}
+	if (*bufp == '#')
+	    continue;
+	if ((cmd = mystrtok(&bufp, " ")) == 0) {
+	    vstream_printf("usage: del key|get key|put key=value|first|next\n");
 	    vstream_fflush(VSTREAM_OUT);
 	    continue;
 	}
 	if (dict_changed_name())
 	    msg_warn("dictionary has changed");
-	key = vstring_str(unescape(keybuf, mystrtok(&bufp, " =")));
+	key = *bufp ? vstring_str(unescape(keybuf, mystrtok(&bufp, " ="))) : 0;
 	value = mystrtok(&bufp, " =");
 	if (strcmp(cmd, "del") == 0 && key && !value) {
 	    if (dict_del(dict, key))
@@ -409,8 +446,22 @@ int     main(int argc, char **argv)
 	} else if (strcmp(cmd, "put") == 0 && key && value) {
 	    dict_put(dict, key, value);
 	    vstream_printf("%s=%s\n", key, value);
+	} else if (strcmp(cmd, "first") == 0 && !key && !value) {
+	    if (dict_seq(dict, DICT_SEQ_FUN_FIRST, &key, &value) == 0)
+		vstream_printf("%s=%s\n", key, value);
+	    else
+		vstream_printf("%s\n",
+			       dict_errno == DICT_ERR_RETRY ?
+			       "soft error" : "not found");
+	} else if (strcmp(cmd, "next") == 0 && !key && !value) {
+	    if (dict_seq(dict, DICT_SEQ_FUN_NEXT, &key, &value) == 0)
+		vstream_printf("%s=%s\n", key, value);
+	    else
+		vstream_printf("%s\n",
+			       dict_errno == DICT_ERR_RETRY ?
+			       "soft error" : "not found");
 	} else {
-	    vstream_printf("usage: del key|get key|put key=value\n");
+	    vstream_printf("usage: del key|get key|put key=value|first|next\n");
 	}
 	vstream_fflush(VSTREAM_OUT);
     }
